pub struct RefEntry<'a, K, V> { /* private fields */ }Expand description
A reference-counted entry in a skip list.
You must call release to free this type, otherwise the node will be
leaked. This is because releasing the entry requires a Guard.
Implementations
Sourceimpl<'a, K: 'a, V: 'a> RefEntry<'a, K, V>
impl<'a, K: 'a, V: 'a> RefEntry<'a, K, V>
Sourcepub fn is_removed(&self) -> bool
pub fn is_removed(&self) -> bool
Returns true if the entry is removed from the skip list.
Sourcepub fn release_with_pin<F>(self, pin: F)
pub fn release_with_pin<F>(self, pin: F)
Releases the reference of the entry, pinning the thread only when the reference count of the node becomes 0.